# @author Aad Schippers <aad@solidam.com>

require 'curb'
require 'open-uri'
require_relative 'string_helpers'

class GoogleTranslator

	EMPTY_GOOGLE_STRING = "[ [[\"#{__FILE__} error: empty or erroneous response from Google Translate\"]], [] ]"

	attr_accessor :input, :input_language, :output_language, :google_array, :google_string
	attr_accessor :output, :locale

	def initalize
		self.locale = 'nl'
	end

	def translate input, input_language='auto', output_language=locale

		self.input           = input
		self.input_language  = input_language
		self.output_language = output_language

		ask_google

		# return the untranslated input when it is translated to white space only
		self.output = translation.match( /^\s*$/ ) ? input : StringHelpers::equalize( translation, input )
	end

	def language input

		self.input           = input
		self.input_language  = 'auto'
		self.output_language = locale

		ask_google

		google_array[2]
	end

	def translation

		google_array[0][0][0]
	end

	def ask_google

		query_google

		# Google translate returns a javascript array, which may contain empty
		# items, i.e. array = [1,,2] is correct javascript
		# fill empty places of array [1,,   ,4, ,  ] => "[23,-1,-1,4,-1,-1]"
		array_string = google_string.gsub( /,\s*,/, ',-1,' ).gsub( /,\s*,/, ',-1,' ).gsub( /\[\s*,/, '[-1,' ).gsub( /,\s*\]/, ',-1]' )

		self.google_array = eval( array_string )
	end

	def url
		"http://translate.google.nl/translate_a/t"
	end

	def uri_encoded

		URI::encode( input )
	end

	def query
		"?client=t&sl=#{input_language}&tl=#{output_language}&hl=en&sc=2&ie=UTF-8&oe=UTF-8&prev=btn&ssel=0&tsel=0&q=#{uri_encoded}"
	end

	def query_google

		self.google_string = Curl.get( url + query ).body_str || EMPTY_GOOGLE_STRING
	end
end

# Quick Testing
=begin
	#english     = "'none' and 'ad' Articles"
	english     = "Save"
	translator  = GoogleTranslator.new
	translation = translator.translate english, 'en', 'nl'
	puts translation
=end
